;;; sleep.el --- run hooks on sleep and wake  -*- lexical-binding:t -*-

;; Copyright (C) 2023 Free Software Foundation, Inc.

;; Author: Andrew Cohen <>
;; Maintainer: emacs-devel@gnu.org
;; Keywords:

;; This file is part of GNU Emacs.

;; GNU Emacs is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:

;;; This global minor mode enables evaluating code when the device
;;; running Emacs enters or leaves the sleep state.  Two hooks are
;;; used, sleep-sleep-hook and sleep-wake-hook, run when the system
;;; detects that it is going to sleep or waking up.  Currently only a
;;; dbus interface to detect sleep state change is implemented.

;;; Code:

(require 'dbus)

(defgroup sleep nil
  "Run hooks on entering/leaving the sleep state."
  :group 'hardware)

(defcustom sleep-sleep-hook nil
  "Hook to run on entering sleep."
  :group 'sleep
  :type 'hook)

(defcustom sleep-wake-hook nil
  "Hook to run on leaving sleep."
  :group 'sleep
  :type 'hook)

(defvar sleep-registration-object nil
  "Object returned from `dbus-register-signal'.
This is used to unregister the signal.")


(defun sleep-wake-enable ()
  "Enable detection of sleep/wake state change.
Run sleep-sleep-hook and sleep-wake-hook as appropriate."
  (unless sleep-registration-object
    (setq sleep-registration-object
          (dbus-register-signal :system
                                "org.freedesktop.login1"
                                "/org/freedesktop/login1"
                                "org.freedesktop.login1.Manager"
                                "PrepareForSleep"
                                #'sleep-handler))))

(defun sleep-wake-disable ()
  "Disable detection of sleep/wake state change."
  (condition-case nil
      (progn
        (dbus-unregister-object
         sleep-registration-object)
        (setq sleep-registration-object nil))
    (wrong-type-argument nil)))

(defun sleep-handler (sleep-wake)
  "Handler to execute sleep and wake functions.
SLEEP-WAKE is t on sleeping and nil on waking."
  (ignore-errors
    (if sleep-wake
        (run-hooks 'sleep-sleep-hook)
      (run-hooks 'sleep-wake-hook))))

;;;###autoload
(define-minor-mode sleep-wake-mode
  "Toggle sleep/wake detection.

When sleep-wake-mode is enabled, Emacs will execute the hooks
`sleep-sleep-hook' and `sleep-wake-hook' when entering or leaving
the sleep state.  This is currently only available on systems that
support dbus detection of sleep state change."
  :global t
  :group 'sleep
  (when (and (featurep 'dbusbind)
             (member "org.freedesktop.login1"
                     (dbus-list-activatable-names :system)))
    (if sleep-wake-mode
        (sleep-wake-enable)
      (sleep-wake-disable))))

(provide 'sleep)
;;; sleep.el ends here
